import 'package:dotted_border/dotted_border.dart';
import 'package:flutter/material.dart';
import 'package:senior_design_system/senior_design_system.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';

enum SeniorSquareButtonsMenuItemType {
  primary,
  secondary,
  ghost,
}

class SeniorSquareButtonsMenuItemData {
  final Key? key;
  final IconData icon;
  final String text;
  final SeniorSquareButtonsMenuItemType type;
  final Function() onTap;
  final bool disabled;

  late final bool withCheckbox;
  late final bool isChecked;
  late final Function(bool?) onCheck;

  SeniorSquareButtonsMenuItemData({
    required this.icon,
    required this.text,
    required this.type,
    required this.onTap,
    this.disabled = false,
    this.key,
  }) {
    this.withCheckbox = false;
    this.isChecked = false;
    this.onCheck = (_) {};
  }

  SeniorSquareButtonsMenuItemData.checkbox({
    required this.icon,
    required this.text,
    required this.type,
    required this.onTap,
    required this.onCheck,
    required this.isChecked,
    this.disabled = false,
    this.key,
  }) {
    this.withCheckbox = true;
  }
}

class SeniorSquareButtonsMenu extends StatelessWidget {
  final List<SeniorSquareButtonsMenuItemData> items;

  final EdgeInsetsGeometry padding;

  const SeniorSquareButtonsMenu({
    required this.items,
    this.padding = const EdgeInsets.all(4.0),
    Key? key,
  }) : super(key: key);

  factory SeniorSquareButtonsMenu.grid({
    Key? key,
    required List<SeniorSquareButtonsMenuItemData> items,
    int crossAxisCount,
    double crossAxisSpacing,
    double mainAxisSpacing,
  }) = _SeniorSquadButtonsMenuGrid;

  Color _getBackgroundColor({
    required SeniorSquareButtonsMenuItemType type,
    bool disabled = false,
  }) {
    switch (type) {
      case SeniorSquareButtonsMenuItemType.primary:
        return SeniorColors.primaryColor.withOpacity(
          disabled ? 0.3 : 1.0,
        );
      case SeniorSquareButtonsMenuItemType.secondary:
        return SeniorColors.secondaryColor600.withOpacity(
          disabled ? 0.3 : 1.0,
        );
      case SeniorSquareButtonsMenuItemType.ghost:
        return Colors.transparent;
    }
  }

  Color _getDrawingColor(SeniorSquareButtonsMenuItemType type) {
    switch (type) {
      case SeniorSquareButtonsMenuItemType.ghost:
        return SeniorColors.secondaryColor600;
      default:
        return SeniorColors.pureWhite;
    }
  }

  Widget buildMenuItem({
    required SeniorSquareButtonsMenuItemData item,
    required double itemsSize,
  }) {
    final double iconSize = itemsSize * 0.3;
    final bool smallSize = itemsSize < 100.0;

    return Container(
      height: itemsSize,
      width: itemsSize,
      child: Tooltip(
        message: item.text,
        padding: EdgeInsets.zero,
        child: Material(
          color: Colors.transparent,
          child: InkWell(
            borderRadius: BorderRadius.circular(SeniorRadius.xbig),
            onTap: item.onTap,
            child: DottedBorder(
              borderType: BorderType.RRect,
              radius: Radius.circular(SeniorRadius.xbig),
              padding: const EdgeInsets.all(0),
              color: item.type == SeniorSquareButtonsMenuItemType.ghost
                  ? SeniorColors.secondaryColor600
                  : Colors.transparent,
              child: Ink(
                padding: const EdgeInsets.all(8.0),
                decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(SeniorRadius.xbig),
                  gradient: item.type == SeniorSquareButtonsMenuItemType.primary
                      ? LinearGradient(
                          begin: Alignment.bottomLeft,
                          end: Alignment.topRight,
                          colors: SeniorColors.primaryGradientColors,
                        )
                      : null,
                  color: _getBackgroundColor(
                    type: item.type,
                    disabled: item.disabled,
                  ),
                ),
                child: Stack(
                  alignment: Alignment.center,
                  children: [
                    Visibility(
                      visible: item.withCheckbox && !smallSize,
                      child: Align(
                        alignment: Alignment.topLeft,
                        child: Container(
                          decoration: BoxDecoration(
                            borderRadius:
                                BorderRadius.circular(SeniorRadius.small),
                          ),
                          child: SeniorCheckbox(
                            value: item.isChecked,
                            onChanged: item.onCheck,
                          ),
                        ),
                      ),
                    ),
                    Container(
                      child: Column(
                        mainAxisAlignment: !smallSize
                            ? MainAxisAlignment.center
                            : MainAxisAlignment.center,
                        crossAxisAlignment: CrossAxisAlignment.center,
                        children: [
                          Row(
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              Icon(
                                item.icon,
                                color: _getDrawingColor(item.type),
                                size: iconSize,
                              ),
                            ],
                          ),
                          Visibility(
                            visible: !smallSize,
                            child: Padding(
                              padding: EdgeInsets.only(
                                top: itemsSize * 0.12,
                              ),
                              child: Text(
                                item.text,
                                textAlign: TextAlign.center,
                                overflow: TextOverflow.ellipsis,
                                maxLines: 2,
                                style: SeniorTypography.small(
                                  color: _getDrawingColor(item.type),
                                ),
                              ),
                            ),
                          ),
                        ],
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      height: 100,
      child: ListView.separated(
        padding: EdgeInsets.symmetric(
          horizontal: SeniorSpacing.normal,
        ),
        itemCount: items.length,
        scrollDirection: Axis.horizontal,
        separatorBuilder: (context, index) {
          return SizedBox(
            width: SeniorSpacing.xsmall,
          );
        },
        itemBuilder: (context, index) {
          final item = items[index];
          final double itemsSize = 100.0;
          return buildMenuItem(
            item: item,
            itemsSize: itemsSize,
          );
        },
      ),
    );
  }
}

class _SeniorSquadButtonsMenuGrid extends SeniorSquareButtonsMenu {
  late final int crossAxisCount;
  final double crossAxisSpacing;
  final double mainAxisSpacing;

  _SeniorSquadButtonsMenuGrid({
    Key? key,
    required List<SeniorSquareButtonsMenuItemData> items,
    int crossAxisCount = 3,
    this.crossAxisSpacing = SeniorSpacing.xsmall,
    this.mainAxisSpacing = SeniorSpacing.xsmall,
  }) : super(key: key, items: items) {
    this.crossAxisCount = crossAxisCount <= 6 ? crossAxisCount : 6;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
          double menuWidth = constraints.constrainWidth();
          int slots = items.length;
          while (slots % crossAxisCount != 0) slots++;
          double itemsSize =
              (menuWidth - (8 * (crossAxisCount - 1))) / crossAxisCount;

          return GridView.count(
            shrinkWrap: true,
            crossAxisSpacing: crossAxisSpacing,
            mainAxisSpacing: mainAxisSpacing,
            crossAxisCount: crossAxisCount,
            physics: RangeMaintainingScrollPhysics(),
            children: items
                .map(
                  (item) => buildMenuItem(item: item, itemsSize: itemsSize),
                )
                .toList(),
          );
        },
      ),
    );
  }
}
